// PartBrowserPane.cpp : implementation file
//

#include "stdafx.h"
#include "GMEApp.h"
#include "GMEstd.h"
#include "GMEDoc.h"
#include "GMEOLEData.h"
#include "GMEView.h"
#include "GuiObject.h"
#include "PartBrowserPane.h"
#include "PartBrowserPaneFrame.h"
#include "GraphicsUtil.h"
#include "GMEEventLogger.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPartBrowserPane

CPartBrowserPane::CPartBrowserPane() : parts(0), guiMetaModel(0), aspect(0)
{
	backgroundColor = ::GetSysColor(COLOR_APPWORKSPACE);
}

CPartBrowserPane::~CPartBrowserPane()
{
}

void CPartBrowserPane::Resize(CRect r)
{
	if(!parts)
		return;

	ChangeAspect(aspect);

	const int leftMargin = 10;
	const int rightMargin = 10;
	const int topMargin = 10;
	const int bottomMargin = 30;

	const int xPadding = 10;
	const int yPadding = 30;

	// set the location
	int objColumn = 0;
	int leftStartPos = leftMargin + maxSize.cx / 2;
	int rowHeight = 0;

	// calculate the actual offset
	int objNumInRow = (r.Width() - leftMargin - rightMargin + xPadding) / (maxSize.cx + xPadding);
	if( objNumInRow < 1 )
	{
		objNumInRow = 1;
		leftStartPos = r.Width() / 2 - 3;
	}

	// the pt contains the x center point, and y top point
	CPoint pt = CPoint(leftStartPos, topMargin);

	POSITION pos = parts->GetHeadPosition();
	while(pos)
	{
		CGuiObject *guiObj = parts->GetNext(pos);
		if( guiObj->IsPrimary(guiMetaModel,aspect) ) 
		{
			CSize size = guiObj->GetSize();

			// calculate the row height
			if( objColumn == 0 )
			{
				rowHeight = size.cy;

				POSITION pos2 = pos;
				while( pos2 && (objColumn + 1) < objNumInRow )
				{
					CGuiObject *guiObj2 = parts->GetNext(pos2);
					if( guiObj2->IsPrimary(guiMetaModel,aspect) ) 
					{
						CSize size = guiObj2->GetSize();

						if( rowHeight < size.cy )
							rowHeight = size.cy;

						++objColumn;
					}
				}

				objColumn = 0;
				pt.y += rowHeight / 2;
			}

			// set the location

			guiObj->SetCenter(pt, aspect, false);	

			if( ++objColumn >= objNumInRow )
			{
				objColumn = 0;
				pt.x = leftStartPos;
				pt.y += rowHeight / 2 + yPadding;
			}
			else
				pt.x += maxSize.cx + xPadding;
		}
	}

	// if no object in this row
	if( objColumn == 0 )
		pt.y -= yPadding;
	else
		pt.y += rowHeight / 2;

	pt.y += bottomMargin;

	CPartBrowserPaneFrame *parent = (CPartBrowserPaneFrame*)GetParent();
	ASSERT( parent != NULL );

	parent->logicalHeight = pt.y;
	parent->pageHeight = r.Height();
	parent->SetScrollBar();

	Invalidate();
}

void CPartBrowserPane::ChangeAspect(int index)
{
	aspect = index;
	if(!parts)
		return;

	// create the text metric object
	CDC textMetric;
	textMetric.CreateCompatibleDC(NULL);
	textMetric.SelectObject(graphics.GetFont(GME_NAME_FONT, 1.0));

	// just to be safe
	maxSize.cx = 30;
	maxSize.cy = 10;

	// calculate the maximum size
	POSITION pos = parts->GetHeadPosition();
	while(pos)
	{
		CGuiObject *guiObj = parts->GetNext(pos);
		guiObj->SetAspect(aspect);
		if( guiObj->IsPrimary(guiMetaModel,aspect) ) 
		{
			CSize size = guiObj->GetSize();

			if( maxSize.cx < size.cx )
				maxSize.cx = size.cx;

			if( maxSize.cy < size.cy )
				maxSize.cy = size.cy;

			int text_cx = textMetric.GetTextExtent(guiObj->name).cx;
			if( maxSize.cx < text_cx )
				maxSize.cx = text_cx;
		}
	}

	CPartBrowserPaneFrame *parent = (CPartBrowserPaneFrame*)GetParent();
	ASSERT( parent != NULL );

	Invalidate();
}

CGuiObject *CPartBrowserPane::FindObject(CPoint &pt)
{
	if(parts) {
		POSITION pos = parts->GetHeadPosition();
		while(pos) {
			CGuiObject *obj = parts->GetNext(pos);
			if(obj && obj->IsPrimary(guiMetaModel,aspect) && obj->IsInside(pt))
				return obj;
		}
	}
	return 0;
}

BEGIN_MESSAGE_MAP(CPartBrowserPane, CWnd)
	//{{AFX_MSG_MAP(CPartBrowserPane)
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CPartBrowserPane message handlers

void CPartBrowserPane::OnPaint() 
{
	CPartBrowserPaneFrame *parent = (CPartBrowserPaneFrame*)GetParent();
	ASSERT( parent != NULL );

	CPaintDC dc(this); // device context for painting
	dc.SetWindowOrg(0, parent->scrollPos);

	if(parts) {
		POSITION pos = parts->GetHeadPosition();
		while(pos) {
			CGuiObject *guiObj = parts->GetNext(pos);
			if(guiObj->IsPrimary(guiMetaModel,aspect))
				guiObj->Draw(&dc);
		}
	}

	// Do not call CWnd::OnPaint() for painting messages
}

BOOL CPartBrowserPane::OnEraseBkgnd(CDC* pDC) 
{
	RECT r;
	GetClientRect(&r);
    pDC->FillSolidRect(&r,backgroundColor);
	return TRUE;
	
//	return CWnd::OnEraseBkgnd(pDC);
}

void CPartBrowserPane::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CGMEEventLogger::LogGMEEvent("CPartBrowserPane::OnLButtonDown\r\n");
	
	// Needed to flush the Object Inspector and the Browser (JIRA #GME-52)
	SetFocus();

	CPartBrowserPaneFrame *parent = (CPartBrowserPaneFrame*)GetParent();
	ASSERT( parent != NULL );

	point.y += parent->scrollPos;

	CGuiObject *guiObj = FindObject(point);

	if(guiObj) {
		CGMEEventLogger::LogGMEEvent("    LButton over "+guiObj->GetName()+"\r\n");
		CGuiObjectList list;
		list.AddTail(guiObj);

		CRect rectAwake = CRect(point.x,point.y,point.x + 1,point.y + 1);
		rectAwake.InflateRect(3,3);
		CRectList rects;
		CGuiObject::GetRectList(list,rects);
		CRect objRect;

		//
		POSITION pos = rects.GetHeadPosition(); CRect first = rects.GetNext(pos);
		objRect = guiObj->GetLocation();
		CPoint  ptClickOffset(point.x - objRect.left, point.y - objRect.top);
		//
		
		CRectList dummyAnnList;
		CGMEDataDescriptor desc(rects,dummyAnnList,point,ptClickOffset);
	
		CGMEDoc::DoDragDrop(guiObj, &desc, DROPEFFECT_COPY, &rectAwake);
	}			

	CWnd::OnLButtonDown(nFlags, point);
}
